<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

class MRWP_Mass_Shipping {

    //Action wp_ajax for expedition creation
    public static function fetch_creation() {
        global $wpdb;

        // Get the order number
        $order_id = intval( $_POST['numcom'] );

        $response['numcom'] = $order_id;

        // Get the Mondial Relay parameters
        $params_compte = class_MRWP_order::get_params_compte();

        // Mondial Relay WebService
        $client = new SoapClient(class_MRWP_main::MONDIALRELAY_WEBSERVICE);

        // Get the Mondial Relay point
        $id_mondial_relay = class_MRWP_db::get_meta_champ($order_id, 'Mondial Relay Parcel Shop ID');

        if ((isset($id_mondial_relay) && $id_mondial_relay)) {
            $explode_id_mondial_relay = explode('-', $id_mondial_relay);
            $params_livraison = class_MRWP_order::get_params_livraison();
            $params_livraison['liv_rel_pays_mondial_relay'] = $explode_id_mondial_relay[0];
            $params_livraison['liv_rel_mondial_relay'] = $explode_id_mondial_relay[1];
        } else {
            $params_livraison['liv_rel_pays_mondial_relay'] = '';
            $params_livraison['liv_rel_mondial_relay'] = '';
        }

        // Get the sender parameters
        $params_expediteur = class_MRWP_order::get_params_expediteur();

        // Get the recipient parameters
        $params_destinataire = class_MRWP_order::get_params_destinataire($order_id);

        // Remove accents
        foreach($params_destinataire as $key => $champ) {
            $params_destinataire[$key] = remove_accents($champ);
        }

        // Split adress if more than 32 characters
        $numchar = strlen($params_destinataire['_shipping_address_1']);
        if ($numchar > 32 ) {
            $numcut = strrpos($params_destinataire['_shipping_address_1'],' ', -($numchar - 30));
            $params_destinataire['_shipping_address_2'] = substr($params_destinataire['_shipping_address_1'], $numcut + 1, $numchar);
            $params_destinataire['_shipping_address_1'] = substr($params_destinataire['_shipping_address_1'], 0, $numcut);
        }

        if (strlen($params_destinataire['_shipping_company']) > 32 ) {
            $params_destinataire['_shipping_company'] = substr($params_destinataire['_shipping_company'], 0, 31);
        }

        // Remove quotes (') on Dest_Ad3 parameter for the Mondial Relay webservice
        $params_destinataire['_shipping_address_1'] = str_replace("&#039;", " ", $params_destinataire['_shipping_address_1']);

        // Remove CRLF codes and specials characters for the Mondial Relay webservice
        $elim = array("'", "°", "’", "¨", "«", "»", "(", ")", "+", "''");

        $params_destinataire['_shipping_full_name'] = str_replace("\n","", $params_destinataire['_shipping_full_name']);
        $params_destinataire['_shipping_full_name'] = str_replace("\r","", $params_destinataire['_shipping_full_name']);
        $params_destinataire['_shipping_full_name'] = str_replace($elim, " ", $params_destinataire['_shipping_full_name']);

        $params_destinataire['_shipping_company'] = str_replace("\n","", $params_destinataire['_shipping_company']);
        $params_destinataire['_shipping_company'] = str_replace("\r","", $params_destinataire['_shipping_company']);
        $params_destinataire['_shipping_company'] = str_replace("°"," ", $params_destinataire['_shipping_company']);
        $params_destinataire['_shipping_company'] = str_replace($elim, " ", $params_destinataire['_shipping_company']);

        $params_destinataire['_shipping_company'] = preg_replace("/@/", ' ', $params_destinataire['_shipping_company']);

        $params_destinataire['_shipping_address_1'] = str_replace("\n","", $params_destinataire['_shipping_address_1']);
        $params_destinataire['_shipping_address_1'] = str_replace("\r","", $params_destinataire['_shipping_address_1']);
        $params_destinataire['_shipping_address_1'] = str_replace($elim, " ", $params_destinataire['_shipping_address_1']);

        $params_destinataire['_shipping_address_2'] = str_replace("\n","", $params_destinataire['_shipping_address_2']);
        $params_destinataire['_shipping_address_2'] = str_replace("\r","", $params_destinataire['_shipping_address_2']);
        $params_destinataire['_shipping_address_2'] = str_replace($elim, " ", $params_destinataire['_shipping_address_2']);

        $params_destinataire['_shipping_city'] = str_replace("\n","", $params_destinataire['_shipping_city']);
        $params_destinataire['_shipping_city'] = str_replace("\r","", $params_destinataire['_shipping_city']);
        $params_destinataire['_shipping_city'] = str_replace($elim, " ", $params_destinataire['_shipping_city']);

        $params_destinataire['_shipping_postcode'] = str_replace("\n","", $params_destinataire['_shipping_postcode']);
        $params_destinataire['_shipping_postcode'] = str_replace("\r","", $params_destinataire['_shipping_postcode']);
        $params_destinataire['_shipping_postcode'] = str_replace($elim, " ", $params_destinataire['_shipping_postcode']);

        $params_destinataire['_shipping_country'] = str_replace("\n","", $params_destinataire['_shipping_country']);
        $params_destinataire['_shipping_country'] = str_replace("\r","", $params_destinataire['_shipping_country']);
        $params_destinataire['_shipping_country'] = str_replace($elim, " ", $params_destinataire['_shipping_country']);

        // Get the order parameters
        $params_commande = class_MRWP_order::get_params_commande();
        $params_commande['ModeCol']  = class_MRWP_main::MODE_COL;
        $params_commande['ModeLiv']  = class_MRWP_db::get_meta_champ($order_id, 'Mondial Relay Shipping Code');
        $params_commande['NDossier'] = $order_id;
        $params_commande['Poids']    = class_MRWP_db::get_meta_champ($order_id, 'Mondial Relay Parcel Weight');

        // Get assurance
        $assurance = get_option('mondialrelay_assurance', class_MRWP_main::ASSURANCE);

        // Get the collect point parameters
        $params_collecte = class_MRWP_order::get_params_collecte();

        // Expedition parameters
        $params = array(
            'Enseigne'       => $params_compte['mondialrelay_code_client'],
            'ModeCol'        => $params_commande['ModeCol'],
            'ModeLiv'        => $params_commande['ModeLiv'],
            'NDossier'       => $params_commande['NDossier'],
            'NClient'        => $params_commande['NClient'],
            'Expe_Langage'   => $params_expediteur['mondialrelay_vendeur_langage'],
            'Expe_Ad1'       => $params_expediteur['mondialrelay_vendeur_adresse1'],
            'Expe_Ad2'       => $params_expediteur['mondialrelay_vendeur_adresse2'],
            'Expe_Ad3'       => $params_expediteur['mondialrelay_vendeur_adresse3'],
            'Expe_Ad4'       => $params_expediteur['mondialrelay_vendeur_adresse4'],
            'Expe_Ville'     => $params_expediteur['mondialrelay_vendeur_ville'],
            'Expe_CP'        => $params_expediteur['mondialrelay_vendeur_cp'],
            'Expe_Pays'      => $params_expediteur['mondialrelay_vendeur_pays'],
            'Expe_Tel1'      => $params_expediteur['mondialrelay_vendeur_tel'],
            'Expe_Tel2'      => $params_expediteur['mondialrelay_vendeur_tel2'],
            'Expe_Mail'      => $params_expediteur['mondialrelay_vendeur_email'],
            'Dest_Langage'   => $params_destinataire['_shipping_langage'],
            'Dest_Ad1'       => $params_destinataire['_shipping_full_name'],
            'Dest_Ad2'       => $params_destinataire['_shipping_company'],
            'Dest_Ad3'       => $params_destinataire['_shipping_address_1'],
            'Dest_Ad4'       => $params_destinataire['_shipping_address_2'],
            'Dest_Ville'     => $params_destinataire['_shipping_city'],
            'Dest_CP'        => $params_destinataire['_shipping_postcode'],
            'Dest_Pays'      => $params_destinataire['_shipping_country'],
            'Dest_Tel1'      => $params_destinataire['_billing_phone'],
            'Dest_Mail'      => $params_destinataire['_billing_email'],
            'Poids'          => $params_commande['Poids'],
            'Longueur'       => $params_commande['Longueur'],
            'Taille'         => $params_commande['Taille'],
            'NbColis'        => $params_commande['NbColis'],
            'CRT_Valeur'     => $params_commande['CRT_Valeur'],
            'CRT_Devise'     => $params_commande['CRT_Devise'],
            'Exp_Valeur'     => $params_commande['Exp_Valeur'],
            'Exp_Devise'     => $params_commande['Exp_Devise'],
            'COL_Rel_Pays'	 => $params_collecte['col_rel_pays_mondial_relay'],
            'COL_Rel'	 	 => $params_collecte['col_rel_mondial_relay'],
            'LIV_Rel_Pays'   => $params_livraison['liv_rel_pays_mondial_relay'],
            'LIV_Rel'        => $params_livraison['liv_rel_mondial_relay'],
            'TAvisage'       => $params_commande['TAvisage'],
            'TReprise'       => $params_commande['TReprise'],
            'Montage'        => $params_commande['Montage'],
            'TRDV'           => $params_commande['TRDV'],
            // 'Instructions'   => $params_commande['Instructions'],
            'Instructions'   => '',
            'Assurance'      => $assurance,
        );

        // Generation of the security key
        $code = implode("", $params);
        $code .= $params_compte['mondialrelay_cle_privee'];
        $params["Security"] = strtoupper(md5($code));

        // Expedition creation

        // Si le num d'expé existe déjà, alors message d'erreur, sinon création d'expédition
        $expe_exist = class_MRWP_db::get_meta_champ($order_id, 'ExpeditionNum');
        if ($expe_exist) {
            $callback = 421;
        } else {
            $expedition = $client->WSI2_CreationExpedition($params)->WSI2_CreationExpeditionResult;
            $callback = $expedition->STAT;
        }

        // Statut code
        if ( $callback != 0 ) {
            $response['success'] = false;
            $response['message'] = 'Erreur ' . $callback . ' : ' . class_MRWP_helpers::statut($callback);
            $response['numexp']  = 0;
        } else {

            // Add the Mondial Relay expedition paramaters in the database
            $champs_bdd = class_MRWP_order::get_expedition_champs($expedition, $params_commande['NbColis']);
            class_MRWP_db::insert_meta_champs($order_id, $champs_bdd);

            // Get the Mondial Relay email
            $email = class_MRWP_order::get_params_emails($order_id);

            // Hook creation after db updated
            do_action('MRWP_after_expedition_creation', $order_id);

            // Update the checkout status
            $checkout_status = get_option( 'mondialrelay_state_order', '' );
            if ($checkout_status) {
                $order = new WC_Order($order_id);
                if (!empty($order)) {
                    $order->update_status($checkout_status);
                }
            }

            // br in html
            $email['message'] = nl2br($email['message']);

            // Check if email send is activated
            if ($email['send'] == 1) {
                $destinataire = $params_destinataire['_billing_email'];
                // Send the Mondial Relay tracking email to the client
                $mail = wc_mail($destinataire, $email['sujet'], $email['message']);
            }

            // Hook creation after db updated
            do_action('MRWP_after_order_status_updated', $order_id);

            $response['success']  = true;
            $response['message'] = 'Création expédition ' . class_MRWP_db::get_meta_champ( $order_id, 'ExpeditionNum' );
            $response['numexp']  = class_MRWP_db::get_meta_champ( $order_id, 'ExpeditionNum' );
        }

        die( json_encode( $response ) );
    }

    //Action wp_ajax for impression
    public static function fetch_impression() {
        $expe = $_POST['numcom'];

        // Get the global Mondial Relay parameters
        $params_compte     = class_MRWP_order::get_params_compte();
        $params_expediteur = class_MRWP_order::get_params_expediteur();

        // SOAP
        $client = new SoapClient(class_MRWP_main::MONDIALRELAY_WEBSERVICE);
        $params = array(
            'Enseigne'    => $params_compte['mondialrelay_code_client'],
            'Expeditions' => $expe,
            'Langue'      => $params_expediteur['mondialrelay_vendeur_langage'],
        );

        // Generation of the security key
        $code  = implode("", $params);
        $code .= $params_compte['mondialrelay_cle_privee'];
        $params['Security'] = strtoupper(md5($code));

        // get the url
        try{
            $etiquette = $client->WSI3_GetEtiquettes($params)->WSI3_GetEtiquettesResult;
        } catch(Exception $e){
            $debug_except .= $e;
        }

        // Return the urls
        echo json_encode( $etiquette );

        die();
    }

    /**
     * Ajax action - Display page
     */
    public static function display() {

        check_ajax_referer( 'mrwp-list-nonce', 'mrwp_list_nonce' );

        $table = new MRWP_Mass_List_Table();
        $table->prepare_items();
        $table->display();

        die();
    }

    /**
     * Ajax action - Refresh table
     */
    public static function display_refresh() {
        $MRWP_mass_table = new MRWP_Mass_List_Table();
        $MRWP_mass_table->ajax_response();
    }

    /**
     * Ajax action - Update the order weight
     *
     * POST parameters:
     * - order_id  Order ID
     * - weight    Weight in grams
     */
    public static function update_weight() {

        check_ajax_referer( 'mrwp-list-nonce', 'mrwp_list_nonce' );

        $order_id = intval( $_POST['order_id'] );
        $weight   = intval( $_POST['weight'] );
        $success  = update_post_meta( $order_id, 'Mondial Relay Parcel Weight', $weight );

        echo json_encode( [
            'order_id' => $order_id,
            'weight'   => $weight,
            'success'  => $success,
        ] );

        die();
    }

    // Action wp_ajax for cancelling shipment
    public static function cancel_expedition() {

        check_ajax_referer( 'mrwp-list-nonce', 'mrwp_list_nonce' );

        $order_id = intval( $_POST['order_id'] );

        // Set Shipping Code to HMR
        $success = update_post_meta( $order_id, 'Mondial Relay Shipping Code', 'HMR' );

        echo json_encode( [
            'order_id' => $order_id,
            'success'  => $success,
        ] );

        die();
    }

}
