<?php

 // Class for everything related to the database

class class_MRWP_db {


/****************************************************************************
 * Everything related to the option table of WordPress 
 ****************************************************************************/
 // Get options values
    public static function get_options_champs($champs) {
        global $wpdb;
        $prefixe = $wpdb->prefix;

        foreach($champs as $champ => $valeur) {
            $champs[$champ] = $wpdb->get_var( "SELECT option_value FROM " . $prefixe . "options WHERE option_name = '$champ'" );
        }
        return $champs;
    }

 // Get single option value
    public static function get_options_champ($champ, $count = null) {
        global $wpdb;
        $prefixe = $wpdb->prefix;

        if ($count) {
            $champ = $wpdb->get_var("SELECT COUNT(*) as total FROM " . $prefixe . "options WHERE option_name = '$champ'");
        }
        else {
            $champ = $wpdb->get_var("SELECT option_value FROM " . $prefixe . "options WHERE option_name = '$champ'");
        }
        return $champ;
    }

 // Insert options values
    public static function insert_options_champs($champs) {
        global $wpdb;
        $prefixe = $wpdb->prefix;

        foreach($champs as $champ => $valeur) {
            $verif_champ = self::get_options_champ($champ, 1);
            if ($verif_champ == 0) {
                $wpdb->insert($prefixe . 'options', array(
                    'option_id'     => NULL,
                    'option_name'    => '' . $champ . '',
                    'option_value'   => '' . $valeur . '',
                    'autoload'    => 'yes'
                ));
            }
        }
        return 1;
    }

 // Update options values
    public static function update_options_champs($champs) {
        foreach($champs as $champ => $valeur) {
            $valeur = trim($valeur);
            $valeur = strip_tags(trim($valeur));
            update_option($champ, $valeur);
        }
        return 1;
    }

 // Reset options values
    public static function reset_options_champs($champs) {
        foreach($champs as $champ) {
            update_option($champ, '');
        }
        return 1;
    }

 // Delete options values
    public static function delete_options_champs($champs) {
        global $wpdb;
        $prefixe = $wpdb->prefix;

        foreach($champs as $champ) {
            $exist = $wpdb->get_var( "SELECT COUNT(*) as total FROM " . $prefixe . "options WHERE option_name = '" . $champ . "'" );
            if ($exist > 0) {
                $wpdb->delete( $prefixe.'options', array( 'option_name' => $champ ) );
            }
        }
        return 1;
    }

/****************************************************************************
 * Everything related to the postmeta table of WordPress 
 ****************************************************************************/

// Get Meta values
    public static function get_meta_champs($id, $champs) {
        global $wpdb;
        $prefixe = $wpdb->prefix;

        foreach($champs as $key => $champ) {
            $champs[$key] = $wpdb->get_var( "SELECT meta_value FROM " . $prefixe . "postmeta WHERE post_id ='$id' AND meta_key = '$key'");
        }
        return $champs;
    }

// Get Single Meta value
    public static function get_meta_champ($id, $key, $count = null) {
        global $wpdb;
        $prefixe = $wpdb->prefix;

        if ($count)
            $value = $wpdb->get_var( "SELECT COUNT(*) FROM " . $prefixe . "postmeta WHERE post_id ='$id' AND meta_key = '$key'");
        else
            $value = $wpdb->get_var( "SELECT meta_value FROM " . $prefixe . "postmeta WHERE post_id ='$id' AND meta_key = '$key'");
        return $value;
    }

// Insert Meta values
    public static function insert_meta_champs($order_id, $champs) {
        global $wpdb;
        $prefixe = $wpdb->prefix;

        foreach ($champs as $key => $champ) {
            $wpdb->insert($prefixe . 'postmeta', array(
                'meta_id'     => NULL,
                'post_id'    => '' . $order_id . '',
                'meta_key'   => '' . $key . '',
                'meta_value'    => '' . $champ . ''
            ));
        }
        return 1;
    }

/****************************************************************************
 * Everything related to the shipping methods of WooCommerce 
 ****************************************************************************/

// Get WooCommerce shipping methods

    public static function get_shipping_methods() {
        global $wpdb;
        $prefixe = $wpdb->prefix;

        $shipping_methods = $wpdb->get_results("SELECT * FROM " . $prefixe . "woocommerce_shipping_zone_methods WHERE is_enabled = 1");
        return $shipping_methods;
    }

// Get WooCommerce shipping zones
    public static function get_shipping_zones() {
        global $wpdb;
        $prefixe = $wpdb->prefix;

        $shipping_zones = $wpdb->get_results("SELECT * FROM " . $prefixe . "woocommerce_shipping_zones");
        return $shipping_zones;
    }

// Get WooCommerce specific shipping zone
    public static function get_shipping_zone($zone_id) {
        global $wpdb;
        $prefixe = $wpdb->prefix;

        $shipping_zone = $wpdb->get_var("SELECT zone_name FROM " . $prefixe . "woocommerce_shipping_zones WHERE zone_id = " . $zone_id);
        return $shipping_zone;
    }

// Get WooCommerce specific shipping method title
    public static function get_shipping_method_title($shipping_zone_name, $shipping_zone_id) {

        $shipping_title = '';

        if ($shipping_zone_name == 'wbs') {
            // Plugin Weight Based Shipping
            $option = $shipping_zone_name . '_' . $shipping_zone_id . '_config';
            $shipping_zone_settings = get_option($option);
            $shipping_title = $shipping_zone_settings["rules"][0]["meta"]["title"];

        } elseif ($shipping_zone_name == 'jem_table_rate') {
            // Plugin Jem table Rate Shipping
            $option_num = get_option('jem_table_rate_sub_shipping_method_id');
            $option = $shipping_zone_name . '_shipping_methods_' . $shipping_zone_id;
            $shipping_zone_settings = get_option($option);

            for ($i = 1; $i <= $option_num; $i++) {
                if (isset($shipping_zone_settings[$i])) {
                    if ($shipping_zone_settings[$i]["method_title"] != '') {
                        $shipping_title = $shipping_zone_settings[$i]["method_title"];
                    }
                }
            }

        } elseif ($shipping_zone_name == 'jem_table_rate_pro') {
                // Plugin Jem table Rate Shipping Pro
                $option_num = get_option('jem_table_rate_pro_sub_shipping_method_id');
                $option = $shipping_zone_name . '_shipping_methods_' . $shipping_zone_id;
                $shipping_zone_settings = get_option($option);
    
                for ($i = 1; $i <= $option_num; $i++) {
                    if (isset($shipping_zone_settings[$i])) {
                        if ($shipping_zone_settings[$i]["method_title"] != '') {
                            $shipping_title = $shipping_zone_settings[$i]["method_title"];
                        }
                    }
                }

        } elseif ($shipping_zone_name == 'flexible_shipping') {
            // Plugin Flexible Shipping from WP Desk
            $optionnum = $shipping_zone_name . '_method_order_' . $shipping_zone_id;
            $shipping_order = get_option($optionnum);
            $option = $shipping_zone_name . '_methods_' . $shipping_zone_id;
            $shipping_zone_settings = get_option($option);
            
            if (is_array($shipping_order)) {
                $shipping_title = $shipping_zone_settings[key($shipping_order)]["method_title"];
            }
            else {
                $shipping_title = 'Flexible Shipping';
            }


        } elseif ($shipping_zone_name == 'tree_table_rate') {
            // Plugin Tree Table Shipping

            $option = 'woocommerce_' . $shipping_zone_name . '_' . $shipping_zone_id . '_settings';
            $shipping_zone_settings = get_option($option);
            $shipping_title = $shipping_zone_settings['label'];

            if ($shipping_title == '') {
                $shipping_title = 'Tree Table Rate';
            }


        } else {
            // Standard Woocomerce and Plugin Table Rate Shipping from Borders Elements
            $option = 'woocommerce_' . $shipping_zone_name . '_' . $shipping_zone_id . '_settings';
            $shipping_zone_settings = get_option($option);
            $shipping_title = $shipping_zone_settings['title'];
        }

        return $shipping_title;
    }

}
